// ISR based polyphonic piano
// interrupt runs at 40kHz
// currently generates 4 squarewave channels that can independently be turned on/off
// and adjusted from ~500Hz - ~2kHz

//https://www.arduino.cc/reference/en/libraries/capacitivesensor/
#include <CapacitiveSensor.h>

#define SPKR0 A0
#define SPKR1 A1
#define SPKR2 A2
#define SPKR3 A3
#define COM  0
#define LED  13

#define BUTTON_TYPE 0  //0-capacitive, 1-tact switches

//https://pages.mtu.edu/~suits/notefreqs.html
#define nC1  523
#define nC1s 554
#define nD1  587
#define nD1s 622
#define nE1  659
#define nF1  698
#define nF1s 740
#define nG1  784
#define nG1s 830
#define nA1  880
#define nA1s 932
#define nB1  988
#define nC2  1046
#define nC2s 1109
#define nD2  1174
#define nD2s 1244
#define nE2  1318
#define nF2  1397
#define nF2s 1480
#define nG2  1568
#define nG2s 1661
#define nA2  1760
#define nA2s 1864
#define nB2  1975
#define nC3  2093

bool f0on,f1on,f2on,f3on=0;
bool f0st,f1st,f2st,f3st=0;
long f0cnt,f1cnt,f2cnt,f3cnt=0;
long f0max,f1max,f2max,f3max=0;

#define testbit(data,bitno) ((data>>bitno)&0x001)    //Byte manipulation macros
#define bit_set(var,bitno) ((var) |= 1 << (bitno))
#define bit_clr(var,bitno) ((var) &= ~(1 << (bitno)))

unsigned int cnt=0;
unsigned char old_pin=0;
unsigned int timeout_max=100;
unsigned int timeout_cnt=0;
unsigned int recal_cnt=0;
unsigned char tmr_avail[]={0,0,0,0};
unsigned int trigger_cnt=0;
unsigned int trigger_min=3;
bool Btn1,Btn2,Btn3,Btn4,Btn5,Btn6,Btn7,Btn8,Btn9,Btn10,Btn11,Btn12,Btn13=0;
long total1,total2,total3,total4,total5,total6,total7,total8,total9,total10,total11,total12,total13=0;
int sensitivity=0;  //40;

CapacitiveSensor   cs_0 = CapacitiveSensor(COM,4);    //C1
CapacitiveSensor   cs_1 = CapacitiveSensor(COM,3);    //C1s
CapacitiveSensor   cs_2 = CapacitiveSensor(COM,2);    //D1
CapacitiveSensor   cs_3 = CapacitiveSensor(COM,5);    //D1s
CapacitiveSensor   cs_4 = CapacitiveSensor(COM,6);    //E1
CapacitiveSensor   cs_5 = CapacitiveSensor(COM,7);    //F1
CapacitiveSensor   cs_6 = CapacitiveSensor(COM,8);    //F1s
CapacitiveSensor   cs_7 = CapacitiveSensor(COM,9);    //G1
CapacitiveSensor   cs_8 = CapacitiveSensor(COM,10);   //G1s
CapacitiveSensor   cs_9 = CapacitiveSensor(COM,11);   //A1
CapacitiveSensor   cs_10 = CapacitiveSensor(COM,12);  //A1s
CapacitiveSensor   cs_11 = CapacitiveSensor(COM,A4);  //B1
CapacitiveSensor   cs_12 = CapacitiveSensor(COM,A5);  //C2

void cal(void){ //calibrates threshold for capacitive touch inputs, call at start
  int s=100;
  long total1,total2,total3,total4,total5,total6,total7,total8,total9,total10,total11,total12,total13=0;
  while(!total1&&!total2&&!total3&&!total4&&!total5&&!total6&&!total7&&!total8&&!total9&&!total10&&!total11&&!total12&&!total13){
    total1 =  cs_0.capacitiveSensor(30);
    total2 =  cs_1.capacitiveSensor(30);
    total3 =  cs_2.capacitiveSensor(30);
    total4 =  cs_3.capacitiveSensor(30);
    total5 =  cs_4.capacitiveSensor(30);
    total6 =  cs_5.capacitiveSensor(30);
    total7 =  cs_6.capacitiveSensor(30);
    total8 =  cs_7.capacitiveSensor(30);
    total9 =  cs_8.capacitiveSensor(30);
    total10 =  cs_9.capacitiveSensor(30);
    total11 =  cs_10.capacitiveSensor(30);
    total12 =  cs_11.capacitiveSensor(30);
    total13 =  cs_12.capacitiveSensor(30);
    total1=(total1>s);
    total2=(total2>s);
    total3=(total3>s);
    total4=(total4>s);
    total5=(total5>s);
    total6=(total6>s);
    total7=(total7>s);
    total8=(total8>s);
    total9=(total9>s);
    total10=(total10>s);
    total11=(total11>s);
    total12=(total12>s);
    total13=(total13>s);
    s-=1;
    delay(10);
  }
  sensitivity=s+30;
}

void readButtons(void){ //read all capacitive touch inputs and store states in global booleans
  total1 =  cs_0.capacitiveSensor(30);
  total2 =  cs_1.capacitiveSensor(30);
  total3 =  cs_2.capacitiveSensor(30);
  total4 =  cs_3.capacitiveSensor(30);
  total5 =  cs_4.capacitiveSensor(30);
  total6 =  cs_5.capacitiveSensor(30);
  total7 =  cs_6.capacitiveSensor(30);
  total8 =  cs_7.capacitiveSensor(30);
  total9 =  cs_8.capacitiveSensor(30);
  total10 =  cs_9.capacitiveSensor(30);
  total11 =  cs_10.capacitiveSensor(30);
  total12 =  cs_11.capacitiveSensor(30);
  total13 =  cs_12.capacitiveSensor(30);
  
  total1=(total1>sensitivity);
  total2=(total2>sensitivity);
  total3=(total3>sensitivity);
  total4=(total4>sensitivity);
  total5=(total5>sensitivity);
  total6=(total6>sensitivity);
  total7=(total7>sensitivity);
  total8=(total8>sensitivity);
  total9=(total9>sensitivity);
  total10=(total10>sensitivity);
  total11=(total11>sensitivity);
  total12=(total12>sensitivity);
  total13=(total13>sensitivity);
  if(total1||total2||total3||total4||total5||total6||total7||total8||total9||total10||total11||total12||total13){
    digitalWrite(LED,HIGH);
  }else{digitalWrite(LED,LOW);}
}

void beep(int channel, int frequency, bool state){  //configures frequency and state (on/off) for 4 squarewave channels
  if(channel==0){
    f0on=state;
    f0max=(double) 40000/ ((double) frequency*2);
  }else if(channel==1){
    f1on=state;
    f1max=(double) 40000/ ((double) frequency*2);
  }else if(channel==2){
    f2on=state;
    f2max=(double) 40000/ ((double) frequency*2);
  }else if(channel==3){
    f3on=state;
    f3max=(double) 40000/ ((double) frequency*2);
  }
}

void init_timer1(){ //Initialize Timer 1
  TCCR1A=0b00000000; //OC1A+B pins disconnected, CTC Mode
  TCCR1B=0b00001001; //No prescaler, CTC Mode
  OCR1A=400;  //Compare value, sets interrupt frequency (40kHz interrupt)
  TIMSK1=0x02; //CMPA1 Interrupt Enable
  asm("sei"); //Enable Global Interrupts
}

ISR(TIMER1_COMPA_vect){ //interrupt on Timer 1 compare A, runs at 40kHz to generate 4 independent squarewave channels
  //PORTB|=0b00100000;  //LED High
  if(f0cnt<f0max){f0cnt++;}else{
    f0cnt=0;
    if(f0on){
      PINC|=0b00000001;
    }else{
      PORTC&=0b11111110;
    }
  }

  if(f1cnt<f1max){f1cnt++;}else{
    f1cnt=0;
    if(f1on){
      PINC|=0b00000010;
    }else{
      PORTC&=0b11111101;
    }
  }

  if(f2cnt<f2max){f2cnt++;}else{
    f2cnt=0;
    if(f2on){
      PINC|=0b00000100;
    }else{
      PORTC&=0b11111011;
    }
  }

  if(f3cnt<f3max){f3cnt++;}else{
    f3cnt=0;
    if(f3on){
      PINC|=0b00001000;
    }else{
      PORTC&=0b11110111;
    }
  }
  //PORTB&=0b11011111;  //LED Low
}

void channel_queue(void){ //allocates/releases 4 squarewave channels (can turn on/off and adjust frequency) on a first come, first served basis
  if(Btn1){
    if(tmr_avail[0]==0&&tmr_avail[1]!=1&&tmr_avail[2]!=1&&tmr_avail[3]!=1){  //start playing note if channel is available
      beep(0,nC1,1);
      tmr_avail[0]=1;
    }else if(tmr_avail[1]==0&&tmr_avail[0]!=1&&tmr_avail[2]!=1&&tmr_avail[3]!=1){
      beep(1,nC1,1);
      tmr_avail[1]=1;
    }else if(tmr_avail[2]==0&&tmr_avail[0]!=1&&tmr_avail[1]!=1&&tmr_avail[3]!=1){
      beep(2,nC1,1);
      tmr_avail[2]=1;
    }else if(tmr_avail[3]==0&&tmr_avail[0]!=1&&tmr_avail[1]!=1&&tmr_avail[2]!=1){
      beep(3,nC1,1);
      tmr_avail[3]=1;
    }
  }else{
    if(tmr_avail[0]==1){  //stop playing note if button released and this channel was being used
      beep(0,nC1,0);
      tmr_avail[0]=0;
    }else if(tmr_avail[1]==1){
      beep(1,nC1,0);
      tmr_avail[1]=0;
    }else if(tmr_avail[2]==1){
      beep(2,nC1,0);
      tmr_avail[2]=0;
    }else if(tmr_avail[3]==1){
      beep(3,nC1,0);
      tmr_avail[3]=0;
    }
  }

  if(Btn2){
    if(tmr_avail[0]==0&&tmr_avail[1]!=2&&tmr_avail[2]!=2&&tmr_avail[3]!=2){  //start playing note if channel is available
      beep(0,nC1s,1);
      tmr_avail[0]=2;
    }else if(tmr_avail[1]==0&&tmr_avail[0]!=2&&tmr_avail[2]!=2&&tmr_avail[3]!=2){
      beep(1,nC1s,1);
      tmr_avail[1]=2;
    }else if(tmr_avail[2]==0&&tmr_avail[0]!=2&&tmr_avail[1]!=2&&tmr_avail[3]!=2){
      beep(2,nC1s,1);
      tmr_avail[2]=2;
    }else if(tmr_avail[3]==0&&tmr_avail[0]!=2&&tmr_avail[1]!=2&&tmr_avail[2]!=2){
      beep(3,nC1s,1);
      tmr_avail[3]=2;
    }
  }else{
    if(tmr_avail[0]==2){  //stop playing note if button released and this channel was being used
      beep(0,nC1s,0);
      tmr_avail[0]=0;
    }else if(tmr_avail[1]==2){
      beep(1,nC1s,0);
      tmr_avail[1]=0;
    }else if(tmr_avail[2]==2){
      beep(2,nC1s,0);
      tmr_avail[2]=0;
    }else if(tmr_avail[3]==2){
      beep(3,nC1s,0);
      tmr_avail[3]=0;
    }
  }

  if(Btn3){
    if(tmr_avail[0]==0&&tmr_avail[1]!=3&&tmr_avail[2]!=3&&tmr_avail[3]!=3){  //start playing note if channel is available
      beep(0,nD1,1);
      tmr_avail[0]=3;
    }else if(tmr_avail[1]==0&&tmr_avail[0]!=3&&tmr_avail[2]!=3&&tmr_avail[3]!=3){
      beep(1,nD1,1);
      tmr_avail[1]=3;
    }else if(tmr_avail[2]==0&&tmr_avail[0]!=3&&tmr_avail[1]!=3&&tmr_avail[3]!=3){
      beep(2,nD1,1);
      tmr_avail[2]=3;
    }else if(tmr_avail[3]==0&&tmr_avail[0]!=3&&tmr_avail[1]!=3&&tmr_avail[2]!=3){
      beep(3,nD1,1);
      tmr_avail[3]=3;
    }
  }else{
    if(tmr_avail[0]==3){  //stop playing note if button released and this channel was being used
      beep(0,nD1,0);
      tmr_avail[0]=0;
    }else if(tmr_avail[1]==3){
      beep(1,nD1,0);
      tmr_avail[1]=0;
    }else if(tmr_avail[2]==3){
      beep(2,nD1,0);
      tmr_avail[2]=0;
    }else if(tmr_avail[3]==3){
      beep(3,nD1,0);
      tmr_avail[3]=0;
    }
  }

  if(Btn4){
    if(tmr_avail[0]==0&&tmr_avail[1]!=4&&tmr_avail[2]!=4&&tmr_avail[3]!=4){  //start playing note if channel is available
      beep(0,nD1s,1);
      tmr_avail[0]=4;
    }else if(tmr_avail[1]==0&&tmr_avail[0]!=4&&tmr_avail[2]!=4&&tmr_avail[3]!=4){
      beep(1,nD1s,1);
      tmr_avail[1]=4;
    }else if(tmr_avail[2]==0&&tmr_avail[0]!=4&&tmr_avail[1]!=4&&tmr_avail[3]!=4){
      beep(2,nD1s,1);
      tmr_avail[2]=4;
    }else if(tmr_avail[3]==0&&tmr_avail[0]!=4&&tmr_avail[1]!=4&&tmr_avail[2]!=4){
      beep(3,nD1s,1);
      tmr_avail[3]=4;
    }
  }else{
    if(tmr_avail[0]==4){  //stop playing note if button released and this channel was being used
      beep(0,nD1s,0);
      tmr_avail[0]=0;
    }else if(tmr_avail[1]==4){
      beep(1,nD1s,0);
      tmr_avail[1]=0;
    }else if(tmr_avail[2]==4){
      beep(2,nD1s,0);
      tmr_avail[2]=0;
    }else if(tmr_avail[3]==4){
      beep(3,nD1s,0);
      tmr_avail[3]=0;
    }
  }

  if(Btn5){
      if(tmr_avail[0]==0&&tmr_avail[1]!=5&&tmr_avail[2]!=5&&tmr_avail[3]!=5){  //start playing note if channel is available
        beep(0,nE1,1);
        tmr_avail[0]=5;
      }else if(tmr_avail[1]==0&&tmr_avail[0]!=5&&tmr_avail[2]!=5&&tmr_avail[3]!=5){
        beep(1,nE1,1);
        tmr_avail[1]=5;
      }else if(tmr_avail[2]==0&&tmr_avail[0]!=5&&tmr_avail[1]!=5&&tmr_avail[3]!=5){
        beep(2,nE1,1);
        tmr_avail[2]=5;
      }else if(tmr_avail[3]==0&&tmr_avail[0]!=5&&tmr_avail[1]!=5&&tmr_avail[2]!=5){
        beep(3,nE1,1);
        tmr_avail[3]=5;
      }
    }else{
      if(tmr_avail[0]==5){  //stop playing note if button released and this channel was being used
        beep(0,nE1,0);
        tmr_avail[0]=0;
      }else if(tmr_avail[1]==5){
        beep(1,nE1,0);
        tmr_avail[1]=0;
      }else if(tmr_avail[2]==5){
        beep(2,nE1,0);
        tmr_avail[2]=0;
      }else if(tmr_avail[3]==5){
        beep(3,nE1,0);
        tmr_avail[3]=0;
      }
    }

  if(Btn6){
    if(tmr_avail[0]==0&&tmr_avail[1]!=6&&tmr_avail[2]!=6&&tmr_avail[3]!=6){  //start playing note if channel is available
      beep(0,nF1,1);
      tmr_avail[0]=6;
    }else if(tmr_avail[1]==0&&tmr_avail[0]!=6&&tmr_avail[2]!=6&&tmr_avail[3]!=6){
      beep(1,nF1,1);
      tmr_avail[1]=6;
    }else if(tmr_avail[2]==0&&tmr_avail[0]!=6&&tmr_avail[1]!=6&&tmr_avail[3]!=6){
      beep(2,nF1,1);
      tmr_avail[2]=6;
    }else if(tmr_avail[3]==0&&tmr_avail[0]!=6&&tmr_avail[1]!=6&&tmr_avail[2]!=6){
      beep(3,nF1,1);
      tmr_avail[3]=6;
    }
  }else{
    if(tmr_avail[0]==6){  //stop playing note if button released and this channel was being used
      beep(0,nF1,0);
      tmr_avail[0]=0;
    }else if(tmr_avail[1]==6){
      beep(1,nF1,0);
      tmr_avail[1]=0;
    }else if(tmr_avail[2]==6){
      beep(2,nF1,0);
      tmr_avail[2]=0;
    }else if(tmr_avail[3]==6){
      beep(3,nF1,0);
      tmr_avail[3]=0;
    }
  }

  if(Btn7){
    if(tmr_avail[0]==0&&tmr_avail[1]!=7&&tmr_avail[2]!=7&&tmr_avail[3]!=7){  //start playing note if channel is available
      beep(0,nF1s,1);
      tmr_avail[0]=7;
    }else if(tmr_avail[1]==0&&tmr_avail[0]!=7&&tmr_avail[2]!=7&&tmr_avail[3]!=7){
      beep(1,nF1s,1);
      tmr_avail[1]=7;
    }else if(tmr_avail[2]==0&&tmr_avail[0]!=7&&tmr_avail[1]!=7&&tmr_avail[3]!=7){
      beep(2,nF1s,1);
      tmr_avail[2]=7;
    }else if(tmr_avail[3]==0&&tmr_avail[0]!=7&&tmr_avail[1]!=7&&tmr_avail[2]!=7){
      beep(3,nF1s,1);
      tmr_avail[3]=7;
    }
  }else{
    if(tmr_avail[0]==7){  //stop playing note if button released and this channel was being used
      beep(0,nF1s,0);
      tmr_avail[0]=0;
    }else if(tmr_avail[1]==7){
      beep(1,nF1s,0);
      tmr_avail[1]=0;
    }else if(tmr_avail[2]==7){
      beep(2,nF1s,0);
      tmr_avail[2]=0;
    }else if(tmr_avail[3]==7){
      beep(3,nF1s,0);
      tmr_avail[3]=0;
    }
  }

  if(Btn8){
    if(tmr_avail[0]==0&&tmr_avail[1]!=8&&tmr_avail[2]!=8&&tmr_avail[3]!=8){  //start playing note if channel is available
      beep(0,nG1,1);
      tmr_avail[0]=8;
    }else if(tmr_avail[1]==0&&tmr_avail[0]!=8&&tmr_avail[2]!=8&&tmr_avail[3]!=8){
      beep(1,nG1,1);
      tmr_avail[1]=8;
    }else if(tmr_avail[2]==0&&tmr_avail[0]!=8&&tmr_avail[1]!=8&&tmr_avail[3]!=8){
      beep(2,nG1,1);
      tmr_avail[2]=8;
    }else if(tmr_avail[3]==0&&tmr_avail[0]!=8&&tmr_avail[1]!=8&&tmr_avail[2]!=8){
      beep(3,nG1,1);
      tmr_avail[3]=8;
    }
  }else{
    if(tmr_avail[0]==8){  //stop playing note if button released and this channel was being used
      beep(0,nG1,0);
      tmr_avail[0]=0;
    }else if(tmr_avail[1]==8){
      beep(1,nG1,0);
      tmr_avail[1]=0;
    }else if(tmr_avail[2]==8){
      beep(2,nG1,0);
      tmr_avail[2]=0;
    }else if(tmr_avail[3]==8){
      beep(3,nG1,0);
      tmr_avail[3]=0;
    }
  }

  if(Btn9){
    if(tmr_avail[0]==0&&tmr_avail[1]!=9&&tmr_avail[2]!=9&&tmr_avail[3]!=9){  //start playing note if channel is available
      beep(0,nG1s,1);
      tmr_avail[0]=9;
    }else if(tmr_avail[1]==0&&tmr_avail[0]!=9&&tmr_avail[2]!=9&&tmr_avail[3]!=9){
      beep(1,nG1s,1);
      tmr_avail[1]=9;
    }else if(tmr_avail[2]==0&&tmr_avail[0]!=9&&tmr_avail[1]!=9&&tmr_avail[3]!=9){
      beep(2,nG1s,1);
      tmr_avail[2]=9;
    }else if(tmr_avail[3]==0&&tmr_avail[0]!=9&&tmr_avail[1]!=9&&tmr_avail[2]!=9){
      beep(3,nG1s,1);
      tmr_avail[3]=9;
    }
  }else{
    if(tmr_avail[0]==9){  //stop playing note if button released and this channel was being used
      beep(0,nG1s,0);
      tmr_avail[0]=0;
    }else if(tmr_avail[1]==9){
      beep(1,nG1s,0);
      tmr_avail[1]=0;
    }else if(tmr_avail[2]==9){
      beep(2,nG1s,0);
      tmr_avail[2]=0;
    }else if(tmr_avail[3]==9){
      beep(3,nG1s,0);
      tmr_avail[3]=0;
    }
  }

  if(Btn10){
    if(tmr_avail[0]==0&&tmr_avail[1]!=10&&tmr_avail[2]!=10&&tmr_avail[3]!=10){  //start playing note if channel is available
      beep(0,nA1,1);
      tmr_avail[0]=10;
    }else if(tmr_avail[1]==0&&tmr_avail[0]!=10&&tmr_avail[2]!=10&&tmr_avail[3]!=10){
      beep(1,nA1,1);
      tmr_avail[1]=10;
    }else if(tmr_avail[2]==0&&tmr_avail[0]!=10&&tmr_avail[1]!=10&&tmr_avail[3]!=10){
      beep(2,nA1,1);
      tmr_avail[2]=10;
    }else if(tmr_avail[3]==0&&tmr_avail[0]!=10&&tmr_avail[1]!=10&&tmr_avail[2]!=10){
      beep(3,nA1,1);
      tmr_avail[3]=10;
    }
  }else{
    if(tmr_avail[0]==10){  //stop playing note if button released and this channel was being used
      beep(0,nA1,0);
      tmr_avail[0]=0;
    }else if(tmr_avail[1]==10){
      beep(1,nA1,0);
      tmr_avail[1]=0;
    }else if(tmr_avail[2]==10){
      beep(2,nA1,0);
      tmr_avail[2]=0;
    }else if(tmr_avail[3]==10){
      beep(3,nA1,0);
      tmr_avail[3]=0;
    }
  }

  if(Btn11){
    if(tmr_avail[0]==0&&tmr_avail[1]!=11&&tmr_avail[2]!=11&&tmr_avail[3]!=11){  //start playing note if channel is available
      beep(0,nA1s,1);
      tmr_avail[0]=11;
    }else if(tmr_avail[1]==0&&tmr_avail[0]!=11&&tmr_avail[2]!=11&&tmr_avail[3]!=11){
      beep(1,nA1s,1);
      tmr_avail[1]=11;
    }else if(tmr_avail[2]==0&&tmr_avail[0]!=11&&tmr_avail[1]!=11&&tmr_avail[3]!=11){
      beep(2,nA1s,1);
      tmr_avail[2]=11;
    }else if(tmr_avail[3]==0&&tmr_avail[0]!=11&&tmr_avail[1]!=11&&tmr_avail[2]!=11){
      beep(3,nA1s,1);
      tmr_avail[3]=11;
    }
  }else{
    if(tmr_avail[0]==11){  //stop playing note if button released and this channel was being used
      beep(0,nA1s,0);
      tmr_avail[0]=0;
    }else if(tmr_avail[1]==11){
      beep(1,nA1s,0);
      tmr_avail[1]=0;
    }else if(tmr_avail[2]==11){
      beep(2,nA1s,0);
      tmr_avail[2]=0;
    }else if(tmr_avail[3]==11){
      beep(3,nA1s,0);
      tmr_avail[3]=0;
    }
  }

  if(Btn12){
    if(tmr_avail[0]==0&&tmr_avail[1]!=12&&tmr_avail[2]!=12&&tmr_avail[3]!=12){  //start playing note if channel is available
      beep(0,nB1,1);
      tmr_avail[0]=12;
    }else if(tmr_avail[1]==0&&tmr_avail[0]!=12&&tmr_avail[2]!=12&&tmr_avail[3]!=12){
      beep(1,nB1,1);
      tmr_avail[1]=12;
    }else if(tmr_avail[2]==0&&tmr_avail[0]!=12&&tmr_avail[1]!=12&&tmr_avail[3]!=12){
      beep(2,nB1,1);
      tmr_avail[2]=12;
    }else if(tmr_avail[3]==0&&tmr_avail[0]!=12&&tmr_avail[1]!=12&&tmr_avail[2]!=12){
      beep(3,nB1,1);
      tmr_avail[3]=12;
    }
  }else{
    if(tmr_avail[0]==12){  //stop playing note if button released and this channel was being used
      beep(0,nB1,0);
      tmr_avail[0]=0;
    }else if(tmr_avail[1]==12){
      beep(1,nB1,0);
      tmr_avail[1]=0;
    }else if(tmr_avail[2]==12){
      beep(2,nB1,0);
      tmr_avail[2]=0;
    }else if(tmr_avail[3]==12){
      beep(3,nB1,0);
      tmr_avail[3]=0;
    }
  }

  if(Btn13){
    if(tmr_avail[0]==0&&tmr_avail[1]!=13&&tmr_avail[2]!=13&&tmr_avail[3]!=13){  //start playing note if channel is available
      beep(0,nC2,1);
      tmr_avail[0]=13;
    }else if(tmr_avail[1]==0&&tmr_avail[0]!=13&&tmr_avail[2]!=13&&tmr_avail[3]!=13){
      beep(1,nC2,1);
      tmr_avail[1]=13;
    }else if(tmr_avail[2]==0&&tmr_avail[0]!=13&&tmr_avail[1]!=13&&tmr_avail[3]!=13){
      beep(2,nC2,1);
      tmr_avail[2]=13;
    }else if(tmr_avail[3]==0&&tmr_avail[0]!=13&&tmr_avail[1]!=13&&tmr_avail[2]!=13){
      beep(3,nC2,1);
      tmr_avail[3]=13;
    }
  }else{
    if(tmr_avail[0]==13){  //stop playing note if button released and this channel was being used
      beep(0,nC2,0);
      tmr_avail[0]=0;
    }else if(tmr_avail[1]==13){
      beep(1,nC2,0);
      tmr_avail[1]=0;
    }else if(tmr_avail[2]==13){
      beep(2,nC2,0);
      tmr_avail[2]=0;
    }else if(tmr_avail[3]==13){
      beep(3,nC2,0);
      tmr_avail[3]=0;
    }
  }
}

void setup() {
  pinMode(LED,OUTPUT);
  pinMode(SPKR0,OUTPUT);
  pinMode(SPKR1,OUTPUT);
  pinMode(SPKR2,OUTPUT);
  pinMode(SPKR3,OUTPUT);

  if(BUTTON_TYPE==0){
    cal();
  }else{
    for(int i=2;i<13;i++){
      pinMode(i,INPUT_PULLUP);
    }
    pinMode(A4,INPUT_PULLUP);
    pinMode(A5,INPUT_PULLUP);
  }
  init_timer1();
}

void loop() {
	if(BUTTON_TYPE==0){
	  readButtons();

	  Btn1=total1;
	  Btn2=total2;
	  Btn3=total3;
	  Btn4=total4;
	  Btn5=total5;
	  Btn6=total6;
	  Btn7=total7;
	  Btn8=total8;
	  Btn9=total9;
	  Btn10=total10;
	  Btn11=total11;
	  Btn12=total12;
	  Btn13=total13;
	}else{
	  Btn1=!digitalRead(2);
	  Btn2=!digitalRead(3);
	  Btn3=!digitalRead(4);
	  Btn4=!digitalRead(5);
	  Btn5=!digitalRead(6);
	  Btn6=!digitalRead(7);
	  Btn7=!digitalRead(8);
	  Btn8=!digitalRead(9);
	  Btn9=!digitalRead(10);
	  Btn10=!digitalRead(11);
	  Btn11=!digitalRead(12);
	  Btn12=!digitalRead(A4);
	  Btn13=!digitalRead(A5);
	}

	channel_queue();
	delay(25);
}
